# Load the data
hght_wide<-read.table('C:/GRE2016/Data/bgs_height_wide.dat', na.string='.')

# Give the variable names
names(hght_wide)<-c('id', 'hght01', 'hght03', 'hght06', 'hght09', 'hght12', 
                          'hght15', 'hght18', 'hght24', 'hght36')

library(OpenMx)

################# Quadratic Growth Model ################# 
quad.hght.omx <- mxModel("Quadratic Growth Model, Path Specification", type="RAM",
  mxData(observed=hght_wide, type="raw"),
  manifestVars=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
  latentVars=c('eta_1','eta_2','eta_3'),

# residual variances
mxPath(from=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
  arrows=2, free=TRUE, values=10, labels='theta'),

# latent variable variances & covariances
mxPath(from=c('eta_1','eta_2','eta_3'), connect='unique.pairs',
  arrows=2, free=TRUE, 
  values = c(60,0,0,4,0,.3),
  labels=c('psi_11','psi_21','psi_31','psi_22','psi_32','psi_33')),

# factor loadings
mxPath(from='eta_1', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
  arrows=1, free=FALSE, values=1),

mxPath(from='eta_2', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
  arrows=1, free=FALSE, 
  values=c(-1.4167, -1.25, -1, -.75, -.5, -.25, 0, .5, 1.5)),

mxPath(from='eta_3', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
  arrows=1, free=FALSE,
  values=c(2.1169, 1.5625, 1, .5625, .25, .0625, 0, .25, 2.25)),

# latent variable means  
mxPath(from='one', to=c('eta_1','eta_2','eta_3'),
  arrows=1, free=TRUE, 
  values=c(65, 2, -.1),
  labels=c('alpha_1','alpha_2','alpha_3'))

) # Close Model

quad.hght.fit <- mxRun(quad.hght.omx)

summary(quad.hght.fit)

#free parameters:
#      name matrix    row    col   Estimate  Std.Error lbound ubound
#1    theta      S hght01 hght01  2.3252453 0.17611018              
#2   psi_11      S  eta_1  eta_1  8.2558661 1.49586691              
#3   psi_21      S  eta_1  eta_2  1.9665494 0.49996005              
#4   psi_22      S  eta_2  eta_2  0.9148372 0.24177993              
#5   psi_31      S  eta_1  eta_3 -0.4681304 0.29604222              
#6   psi_32      S  eta_2  eta_3 -0.4111299 0.12166910              
#7   psi_33      S  eta_3  eta_3  0.1148382 0.10801672              
#8  alpha_1      M      1  eta_1 83.0745550 0.34331378              
#9  alpha_2      M      1  eta_2 13.7602095 0.13712914              
#10 alpha_3      M      1  eta_3 -3.3893227 0.08903753              
#
#observed statistics:  581 
#estimated parameters:  10 
#degrees of freedom:  571 
#-2 log likelihood:  2446.138 
#saturated -2 log likelihood:  NA 
#number of observations:  83 
#chi-square:  NA 
#p:  NA 
#Information Criteria: 
#     df Penalty Parameters Penalty Sample-Size Adjusted
#AIC: 1304.13751           2466.138                   NA
#BIC:  -77.02047           2490.326             2458.783
#CFI: NA 
#TLI: NA 
#RMSEA:  NA 
#timestamp: 2014-08-28 20:08:02 
#frontend time: 0.522053 secs 
#backend time: 0.112011 secs 
#independent submodels time: 0 secs 
#wall clock time: 0.634064 secs 
#cpu time: 0.634064 secs 
#OpenMx version number: 999.0.0-3473 

mxStandardizeRAMpaths(quad.hght.fit)




################# Adjusted Quadratic Growth Model ################# 
quad.hght.omx <- mxModel("Adjusted Quadratic Growth Model, Path Specification", type="RAM",
  mxData(observed=hght_wide, type="raw"),
  manifestVars=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
  latentVars=c('eta_1','eta_2','eta_3'),

# residual variances
mxPath(from=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
  arrows=2, free=TRUE, values=10, labels='theta'),

# latent variable variances & covariances
mxPath(from=c('eta_1','eta_2'), connect='unique.pairs',
  arrows=2, free=TRUE, 
  values = c(60,0,4),
  labels=c('psi_11','psi_21','psi_22')),

# factor loadings
mxPath(from='eta_1', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
  arrows=1, free=FALSE, values=1),

mxPath(from='eta_2', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
  arrows=1, free=FALSE, 
  values=c(-1.4167, -1.25, -1, -.75, -.5, -.25, 0, .5, 1.5)),

mxPath(from='eta_3', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
  arrows=1, free=FALSE,
  values=c(2.1169, 1.5625, 1, .5625, .25, .0625, 0, .25, 2.25)),

# latent variable means  
mxPath(from='one', to=c('eta_1','eta_2','eta_3'),
  arrows=1, free=TRUE, 
  values=c(65, 2, -.1),
  labels=c('alpha_1','alpha_2','alpha_3'))

) # Close Model

quad.hght.fit <- mxRun(quad.hght.omx)

summary(quad.hght.fit)


#free parameters:
#     name matrix    row    col   Estimate  Std.Error lbound ubound
#1   theta      S hght01 hght01  2.4473261 0.16933375              
#2  psi_11      S  eta_1  eta_1  7.3557848 1.26283433              
#3  psi_21      S  eta_1  eta_2  1.5186423 0.43057501              
#4  psi_22      S  eta_2  eta_2  0.8249283 0.22747881              
#5 alpha_1      M      1  eta_1 83.0084357 0.32322897              
#6 alpha_2      M      1  eta_2 13.7301441 0.13453224              
#7 alpha_3      M      1  eta_3 -3.3945084 0.08180242              
#
#observed statistics:  581 
#estimated parameters:  7 
#degrees of freedom:  574 
#-2 log likelihood:  2465.166 
#saturated -2 log likelihood:  NA 
#number of observations:  83 
#chi-square:  NA 
#p:  NA 
#Information Criteria: 
#     df Penalty Parameters Penalty Sample-Size Adjusted
#AIC: 1317.16592           2479.166                   NA
#BIC:  -71.24859           2496.098             2474.018
#CFI: NA 
#TLI: NA 
#RMSEA:  NA 
#timestamp: 2014-08-28 20:21:18 
#frontend time: 0.025002 secs 
#backend time: 0.04500508 secs 
#independent submodels time: 0 secs 
#wall clock time: 0.07000709 secs 
#cpu time: 0.07000709 secs 
#OpenMx version number: 999.0.0-3473 









################# Spline Growth Model ################# 
spl.hght.omx <- mxModel("Spline Growth Model, Path Specification", type="RAM",
  mxData(observed=hght_wide, type="raw"),
 manifestVars=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),   
 latentVars=c('eta_1','eta_2','eta_3'),

# Residual Variances
mxPath(from=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
  arrows=2, free=TRUE, 
  values=10, labels='theta'),

# Latent Variable Covariances
mxPath(from=c('eta_1','eta_2','eta_3'), connect='unique.pairs',
  arrows=2, free=TRUE, 
  values = c(1,0,0,1,0,1),
  labels=c('psi_11','psi_21','psi_31','psi_22','psi_32','psi_33')),

# Factor Loadings
mxPath(from='eta_1', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
  arrows=1, free=FALSE, values=1),

mxPath(from='eta_2', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
  arrows=1, free=FALSE,
  values=c(-8, -6, -3, 0, 0, 0, 0, 0, 0)),

mxPath(from='eta_3', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
  arrows=1, free=FALSE,
  values=c(0, 0, 0, 0, 3, 6, 9, 15, 27)),

# Latent Variable Means
mxPath(from='one', to=c('eta_1','eta_2','eta_3'),
  arrows=1, free=TRUE, 
  values=c(80, 15, 10), labels=c('alpha_1','alpha_2','alpha_3'))

) # Close Model

spl.hght.fit <- mxRun(spl.hght.omx)

summary(spl.hght.fit) 

#free parameters:
      name matrix    row    col     Estimate   Std.Error lbound ubound
1    theta      S hght01 hght01  1.812020208 0.135543120              
2   psi_11      S  eta_1  eta_1  6.124518507 1.099165247              
3   psi_21      S  eta_1  eta_2  0.233319414 0.095816932              
4   psi_22      S  eta_2  eta_2  0.043629605 0.014042430              
5   psi_31      S  eta_1  eta_3  0.083884733 0.025214984              
6   psi_32      S  eta_2  eta_3  0.008394211 0.002763856              
7   psi_33      S  eta_3  eta_3  0.002173484 0.001085245              
8  alpha_1      M      1  eta_1 73.350596842 0.293940087              
9  alpha_2      M      1  eta_2  2.226691540 0.033347400              
10 alpha_3      M      1  eta_3  0.887424482 0.009501550              

observed statistics:  581 
estimated parameters:  10 
degrees of freedom:  571 
-2 log likelihood:  2339.183 
saturated -2 log likelihood:  NA 
number of observations:  83 
chi-square:  NA 
p:  NA 
Information Criteria: 
     df Penalty Parameters Penalty Sample-Size Adjusted
AIC:  1197.1825           2359.183                   NA
BIC:  -183.9755           2383.371             2351.828
CFI: NA 
TLI: NA 
RMSEA:  NA 
timestamp: 2014-08-28 20:24:29 
frontend time: 0.06100583 secs 
backend time: 0.1070111 secs 
independent submodels time: 0 secs 
wall clock time: 0.1680169 secs 
cpu time: 0.1680169 secs 
OpenMx version number: 999.0.0-3473 

#free parameters:
#      name matrix    row    col     Estimate   Std.Error lbound ubound
#1    theta      S hght01 hght01  1.812020208 0.135543120              
#2   psi_11      S  eta_1  eta_1  6.124518507 1.099165247              
#3   psi_21      S  eta_1  eta_2  0.233319414 0.095816932              
#4   psi_22      S  eta_2  eta_2  0.043629605 0.014042430              
#5   psi_31      S  eta_1  eta_3  0.083884733 0.025214984              
#6   psi_32      S  eta_2  eta_3  0.008394211 0.002763856              
#7   psi_33      S  eta_3  eta_3  0.002173484 0.001085245              
#8  alpha_1      M      1  eta_1 73.350596842 0.293940087              
#9  alpha_2      M      1  eta_2  2.226691540 0.033347400              
#10 alpha_3      M      1  eta_3  0.887424482 0.009501550              
#
#observed statistics:  581 
#estimated parameters:  10 
#degrees of freedom:  571 
#-2 log likelihood:  2339.183 
#saturated -2 log likelihood:  NA 
#number of observations:  83 
#chi-square:  NA 
#p:  NA 
#Information Criteria: 
#     df Penalty Parameters Penalty Sample-Size Adjusted
#AIC:  1197.1825           2359.183                   NA
#BIC:  -183.9755           2383.371             2351.828
#CFI: NA 
#TLI: NA 
#RMSEA:  NA 
#timestamp: 2014-08-28 20:24:29 
#frontend time: 0.06100583 secs 
#backend time: 0.1070111 secs 
#independent submodels time: 0 secs 
#wall clock time: 0.1680169 secs 
#cpu time: 0.1680169 secs 
#OpenMx version number: 999.0.0-3473 





################# Square Root Growth Model ################# 
sqrt.hght.omx <- mxModel("Square Root Growth Model, Path Specification",
 type="RAM", mxData(observed=hght_wide, type="raw"),
 manifestVars=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 latentVars=c('eta_1','eta_2'),

# residual variances 
mxPath(from=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=2, free=TRUE, values=10, labels='v_u'),

# latent variable covariances
mxPath(from=c('eta_1','eta_2'), connect='unique.pairs',
 arrows=2, free=TRUE, values = c(1,0,1), labels=c('psi_11','psi_21','psi_22')),

# Factor Loadings
mxPath(from='eta_1', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE, values=1),

mxPath(from='eta_2', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE, values=c(0, .73205, 1.44949, 2, 2.46410, 2.87298, 3.24264, 3.89898, 5)),

# Latent Variable Means 
mxPath(from='one', to=c('eta_1','eta_2'),
 arrows=1, free=TRUE, values=c(50, 15), labels=c('alpha_1','alpha_2'))

) # Close Model
 
sqrt.hght.fit <- mxRun(sqrt.hght.omx)

summary(sqrt.hght.fit)

#free parameters:
#     name matrix    row    col   Estimate  Std.Error lbound ubound
#1     v_u      S hght01 hght01  0.8962738 0.06200222              
#2  psi_11      S  eta_1  eta_1  5.2111217 0.90542074              
#3  psi_21      S  eta_1  eta_2 -0.2055730 0.20037041              
#4  psi_22      S  eta_2  eta_2  0.4203752 0.08307237              
#5 alpha_1      M      1  eta_1 54.8492003 0.26435002              
#6 alpha_2      M      1  eta_2  8.4254122 0.08113767              
#
#observed statistics:  581 
#estimated parameters:  6 
#degrees of freedom:  575 
#-2 log likelihood:  2039.742 
#saturated -2 log likelihood:  NA 
#number of observations:  83 
#chi-square:  NA 
#p:  NA 
#Information Criteria: 
#     df Penalty Parameters Penalty Sample-Size Adjusted
#AIC:   889.7417           2051.742                   NA
#BIC:  -501.0916           2066.255             2047.329
#CFI: NA 
#TLI: NA 
#RMSEA:  NA 
#timestamp: 2014-08-28 20:30:07 
#frontend time: 0.02500224 secs 
#backend time: 0.05700588 secs 
#independent submodels time: 0 secs 
#wall clock time: 0.08200812 secs 
#cpu time: 0.08200812 secs 
#OpenMx version number: 999.0.0-3473 

#free parameters:
#  name matrix    row    col   Estimate  Std.Error lbound ubound
#1  v_u      S hght01 hght01  0.8962738 0.06200223              
#2  v_0      S   g_0n   g_0n  5.2111173 0.90542307              
#3 c_10      S   g_0n   g_1n -0.2055732 0.20037130              
#4  v_1      S   g_1n   g_1n  0.4203754 0.08307267              
#5 mu_0      M      1   g_0n 54.8491980 0.26434989              
#6 mu_1      M      1   g_1n  8.4254118 0.08113764              

#observed statistics:  581 
#estimated parameters:  6 
#degrees of freedom:  575 
#-2 log likelihood:  2039.742 
#saturated -2 log likelihood:  NA 
#number of observations:  83 
#chi-square:  NA 
#p:  NA 
#Information Criteria: 
#    df Penalty Parameters Penalty Sample-Size Adjusted
#AIC   889.7417           2051.742                   NA
#BIC  -501.0916           2066.255             2047.329
#CFI: NA 
#TLI: NA 
#RMSEA:  NA 
#timestamp: 2012-08-01 13:41:07 
#frontend time: 0.09299994 secs 
#backend time: 0.204 secs 
#independent submodels time: 0 secs 
#wall clock time: 0.2969999 secs 
#cpu time: 0.2969999 secs 
#openmx version number: 1.2.4-2063








