hght_long <- read.table('C:/GRE2016/Data/bgs_height_long.dat', na.string='.')

# Give the variable names
names(hght_long)<-c('id', 'age', 'hght')

summary(hght_long)
attach(hght_long)

library(nlme)

# Quadratic Growth Model
hght.quad.nlme <- nlme(hght~b_1i+b_2i*((age-18)/12)+b_3i*((age-18)/12)^2,
                      data=hght_long,
                      fixed=b_1i+b_2i+b_3i~1,
                      random=b_1i+b_2i+b_3i~1,
                      groups=~id,
                      start=c(30, 10, -3),
                      na.action=na.omit)
summary(hght.quad.nlme) 

#Nonlinear mixed-effects model fit by maximum likelihood
#  Model: hght ~ b_1i + b_2i * ((age - 18)/12) + b_3i * ((age - 18)/12)^2 
# Data: hght_long 
#       AIC      BIC    logLik
#  2505.735 2549.382 -1242.867
#
#Random effects:
# Formula: list(b_1i ~ 1, b_2i ~ 1, b_3i ~ 1)
# Level: id
# Structure: General positive-definite, Log-Cholesky parametrization
#         StdDev    Corr         
#b_1i     2.8721640 b_1i   b_2i  
#b_2i     0.9543463  0.698       
#b_3i     0.4287155 -0.425 -0.944
#Residual 1.5717537              
#
#Fixed effects: b_1i + b_2i + b_3i ~ 1 
#        Value Std.Error  DF   t-value p-value
#b_1i 83.08274 0.3445252 496 241.15142       0
#b_2i 13.80617 0.1380515 496 100.00739       0
#b_3i -3.43110 0.0976621 496 -35.13235       0
# Correlation: 
#     b_1i   b_2i  
#b_2i  0.614       
#b_3i -0.418 -0.405
#
#Standardized Within-Group Residuals:
#        Min          Q1         Med          Q3         Max 
#-2.63104527 -0.62271115  0.06746931  0.64191205  2.59824197 
#
#Number of Observations: 581
#Number of Groups: 83 


# Quadratic Growth Model without Variation in b_3i

hght.quad.nlme <- nlme(hght~b_1i+b_2i*((age-18)/12)+b_3i*((age-18)/12)^2,
                      data=hght_long,
                      fixed=b_1i+b_2i+b_3i~1,
                      random=b_1i+b_2i~1,
                      groups=~id,
                      start=c(30, 10, -3),
                      na.action=na.omit)
summary(hght.quad.nlme)




#Nonlinear mixed-effects model fit by maximum likelihood
#  Model: hght ~ b_1i + b_2i * ((age - 18)/12) + b_3i * ((age - 18)/12)^2 
# Data: hght_long 
#       AIC      BIC    logLik
#  2515.275 2545.828 -1250.637
#
#Random effects:
# Formula: list(b_1i ~ 1, b_2i ~ 1)
# Level: id
# Structure: General positive-definite, Log-Cholesky parametrization
#         StdDev    Corr
#b_1i     2.7040353 b_1i
#b_2i     0.8885105 0.62
#Residual 1.6303875     
#
#Fixed effects: b_1i + b_2i + b_3i ~ 1 
#        Value Std.Error  DF  t-value p-value
#b_1i 83.01636 0.3248686 496 255.5383       0
#b_2i 13.78432 0.1344658 496 102.5117       0
#b_3i -3.42943 0.0870154 496 -39.4118       0
# Correlation: 
#     b_1i   b_2i  
#b_2i  0.532       
#b_3i -0.249  0.010
#
#Standardized Within-Group Residuals:
#       Min         Q1        Med         Q3        Max 
#-2.9850630 -0.5640788  0.1010256  0.6286861  2.8576172 
#
#Number of Observations: 581
#Number of Groups: 83 
 

# Spline Growth Model
hght.spline.nlme <- nlme(hght~b_1i+b_2i*(pmin(0,age-9))+b_3i*(pmax(0,age-9)),
                      data=hght_long,
                      fixed=b_1i+b_2i+b_3i~1,
                      random=b_1i+b_2i+b_3i~1,
                      groups=~id,
                      start=c(60, 10, 6),
                      na.action=na.omit)

summary(hght.spline.nlme)  

#Nonlinear mixed-effects model fit by maximum likelihood
#  Model: hght ~ b_1i + b_2i * (pmin(0, age - 9)) + b_3i * (pmax(0, age -      9)) 
# Data: hght_long 
#       AIC     BIC    logLik
#  2359.183 2402.83 -1169.591
#
#Random effects:
# Formula: list(b_1i ~ 1, b_2i ~ 1, b_3i ~ 1)
# Level: id
# Structure: General positive-definite, Log-Cholesky parametrization
#         StdDev     Corr       
#b_1i     2.47477711 b_1i  b_2i 
#b_2i     0.20887672 0.451      
#b_3i     0.04662073 0.727 0.862
#Residual 1.34611421            
#
#Fixed effects: b_1i + b_2i + b_3i ~ 1 
#        Value  Std.Error  DF   t-value p-value
#b_1i 73.35059 0.29455032 496 249.02568       0
#b_2i  2.22669 0.03329296 496  66.88174       0
#b_3i  0.88742 0.00949461 496  93.46612       0
# Correlation: 
#     b_1i  b_2i 
#b_2i 0.481      
#b_3i 0.208 0.104
#
#Standardized Within-Group Residuals:
#        Min          Q1         Med          Q3         Max 
#-2.71700917 -0.56032901  0.06312586  0.63763210  2.54231673 
#
#Number of Observations: 581
#Number of Groups: 83 





#### Using lme to fit the same models ####

# Quadratic Growth Model
hght.quad.nlme <- nlme(hght~(beta_1+d_1i)+(beta_2+d_2i)*((age-18)/12)+(beta_3+d_3i)*((age-18)/12)^2,
                      data=hght_long,
                      fixed=beta_1+beta_2+beta_3 ~ 1,
                      random=d_1i+d_2i+d_3i ~ 1,
                      groups=~id,
                      start=c(30, 10, -3),
                      na.action=na.omit)

summary(hght.quad.nlme)

#Nonlinear mixed-effects model fit by maximum likelihood
#  Model: hght ~ (beta_1 + d_1i) + (beta_2 + d_2i) * ((age - 18)/12) +      (beta_3 + d_3i) * ((age - 18)/12)^2 
# Data: hght_long 
#       AIC      BIC    logLik
#  2505.735 2549.382 -1242.867
#
#Random effects:
# Formula: list(d_1i ~ 1, d_2i ~ 1, d_3i ~ 1)
# Level: id
# Structure: General positive-definite, Log-Cholesky parametrization
#         StdDev    Corr         
#d_1i     2.8721567 d_1i   d_2i  
#d_2i     0.9543676  0.698       
#d_3i     0.4286808 -0.425 -0.944
#Residual 1.5717509              
#
#Fixed effects: beta_1 + beta_2 + beta_3 ~ 1 
#          Value Std.Error  DF   t-value p-value
#beta_1 83.08275 0.3445240 496 241.15230       0
#beta_2 13.80618 0.1380534 496 100.00604       0
#beta_3 -3.43109 0.0976600 496 -35.13306       0
# Correlation: 
#       beta_1 beta_2
#beta_2  0.614       
#beta_3 -0.418 -0.405
#
#Standardized Within-Group Residuals:
#        Min          Q1         Med          Q3         Max 
#-2.63105565 -0.62271814  0.06750987  0.64190763  2.59819080 
#
#Number of Observations: 581
#Number of Groups: 83 




# Quadratic Growth Model without Variation in b_3i
hght.quad.nlme <- nlme(hght~(beta_1+d_1i)+(beta_2+d_2i)*((age-18)/12)+(beta_3)*((age-18)/12)^2,
                      data=hght_long,
                      fixed=beta_1+beta_2+beta_3 ~ 1,
                      random=d_1i+d_2i~1,
                      groups=~id,
                      start=c(30, 10, -3),
                      na.action=na.omit)

summary(hght.quad.nlme)

#Nonlinear mixed-effects model fit by maximum likelihood
#  Model: hght ~ (beta_1 + d_1i) + (beta_2 + d_2i) * ((age - 18)/12) +      (beta_3) * ((age - 18)/12)^2 
# Data: hght_long 
#       AIC      BIC    logLik
#  2515.275 2545.828 -1250.637
#
#Random effects:
# Formula: list(d_1i ~ 1, d_2i ~ 1)
# Level: id
# Structure: General positive-definite, Log-Cholesky parametrization
#         StdDev    Corr
#d_1i     2.7040353 d_1i
#d_2i     0.8885106 0.62
#Residual 1.6303875     
#
#Fixed effects: beta_1 + beta_2 + beta_3 ~ 1 
#          Value Std.Error  DF  t-value p-value
#beta_1 83.01636 0.3248686 496 255.5383       0
#beta_2 13.78432 0.1344658 496 102.5117       0
#beta_3 -3.42943 0.0870154 496 -39.4118       0
# Correlation: 
#       beta_1 beta_2
#beta_2  0.532       
#beta_3 -0.249  0.010
#
#Standardized Within-Group Residuals:
#       Min         Q1        Med         Q3        Max 
#-2.9850630 -0.5640788  0.1010256  0.6286861  2.8576172 
#
#Number of Observations: 581
#Number of Groups: 83 


hght_long$pre = pmin(0,hght_long$age-9)
hght_long$post = pmax(0,hght_long$age-9)

hght.spline.nlme <- nlme(hght~b_1i+b_2i*pre+b_3i*post,
                      data=hght_long,
                      fixed=b_1i+b_2i+b_3i~1,
                      random=b_1i+b_2i+b_3i~1,
                      groups=~id,
                      start=c(60, 10, 6),
                      na.action=na.omit)

summary(hght.spline.nlme)

#Nonlinear mixed-effects model fit by maximum likelihood
#  Model: hght ~ b_1i + b_2i * pre + b_3i * post 
# Data: hght_long 
#       AIC     BIC    logLik
#  2359.183 2402.83 -1169.591
#
#Random effects:
# Formula: list(b_1i ~ 1, b_2i ~ 1, b_3i ~ 1)
# Level: id
# Structure: General positive-definite, Log-Cholesky parametrization
#         StdDev     Corr       
#b_1i     2.47477711 b_1i  b_2i 
#b_2i     0.20887672 0.451      
#b_3i     0.04662073 0.727 0.862
#Residual 1.34611421            
#
#Fixed effects: b_1i + b_2i + b_3i ~ 1 
#        Value  Std.Error  DF   t-value p-value
#b_1i 73.35059 0.29455032 496 249.02568       0
#b_2i  2.22669 0.03329296 496  66.88174       0
#b_3i  0.88742 0.00949461 496  93.46612       0
# Correlation: 
#     b_1i  b_2i 
#b_2i 0.481      
#b_3i 0.208 0.104
#
#Standardized Within-Group Residuals:
#        Min          Q1         Med          Q3         Max 
#-2.71700917 -0.56032901  0.06312586  0.63763210  2.54231673 
#
#Number of Observations: 581
#Number of Groups: 83 










