ODS GRAPHICS OFF;
ODS HTML CLOSE;
ODS LISTING;

DATA height_long;
	INFILE 'C:\GRE2016\Data\bgs_height_long.dat';
	INPUT id age hght;
RUN;

GOPTIONS
	FTEXT = ARIAL
	CTEXT = BLACK
	HTEXT = 1.5;
PROC GPLOT DATA = height_long;
	TITLE1 'IGS Height by Age';
	SYMBOL
		Interpol=join
		Color=black
		Repeat=5000
		Value=dot
		Height=.5
		Width=1;
	AXIS1
		LABEL = (F=ARIAL H = 2 A=90 'Height (cm)')
		ORDER = (40 to 110 BY 10)
		MINOR = none
		OFFSET= (5 pct 5 pct);
	AXIS2
		LABEL = (F=ARIAL H = 2 'Age in Months')	
		ORDER = (0 to 40 BY 10)
		MINOR = none
		OFFSET= (5 PCT, 5 PCT);
	PLOT hght * age = id/NOLEGEND VAXIS= AXIS1 HAXIS=AXIS2;
RUN;





DATA height_long;
	SET height_long;
	t1 = (age - 18)/12;
	t2 = ((age - 18)/12)**2;

	logt = LOG(age);
	sqrt = SQRT(age) - 1;

	pre = MIN(0,age - 9);
	post = MAX(0,age - 9);
RUN;

*Linear Model;
PROC MIXED DATA = height_long COVTEST NOCLPRINT;
	CLASS id;
	MODEL hght = t1/SOLUTION DDFM=BW;
	RANDOM INTERCEPT t1/SUBJECT=id TYPE=UN GCORR;
RUN;
/*
                                 Covariance Parameter Estimates

                                                   Standard         Z
               Cov Parm     Subject    Estimate       Error     Value        Pr Z

               UN(1,1)      id           6.4243      1.4046      4.57      <.0001
               UN(2,1)      id           1.1652      0.5074      2.30      0.0216
               UN(2,2)      id                0           .       .         .
               Residual                 11.8737      0.7529     15.77      <.0001


                                         Fit Statistics

                              -2 Res Log Likelihood          3194.3
                              AIC (smaller is better)        3200.3
                              AICC (smaller is better)       3200.3
                              BIC (smaller is better)        3207.5

                                   Solution for Fixed Effects

                                         Standard
                Effect       Estimate       Error      DF    t Value    Pr > |t|

                Intercept     79.7043      0.3295      82     241.87      <.0001
                t1            13.7634      0.1687     497      81.57      <.0001
*/

*Quadratic Model;
PROC MIXED DATA = height_long COVTEST NOCLPRINT METHOD=ML;
	CLASS id;
	MODEL hght = t1 t2/SOLUTION DDFM=BW;
	RANDOM INTERCEPT t1 t2/SUBJECT=id TYPE=UN GCORR;
RUN;

/*
                                 Covariance Parameter Estimates

                                                   Standard         Z
               Cov Parm     Subject    Estimate       Error     Value        Pr Z

               UN(1,1)      id           8.1497      1.4958      5.45      <.0001
               UN(2,1)      id           1.9442      0.5028      3.87      0.0001
               UN(2,2)      id           0.8915      0.2461      3.62      0.0001
               UN(3,1)      id          -0.4160      0.2990     -1.39      0.1641
               UN(3,2)      id          -0.4141      0.1246     -3.32      0.0009
               UN(3,3)      id          0.05875      0.1118      0.53      0.2996
               Residual                  2.5792      0.1955     13.20      <.0001


                                         Fit Statistics

                              -2 Log Likelihood              2483.2
                              AIC (smaller is better)        2503.2
                              AICC (smaller is better)       2503.6
                              BIC (smaller is better)        2527.4

                                   Solution for Fixed Effects

                                         Standard
                Effect       Estimate       Error      DF    t Value    Pr > |t|

                Intercept     83.0759      0.3423      82     242.73      <.0001
                t1            13.8161      0.1371     496     100.77      <.0001
                t2            -3.4210     0.08914     496     -38.38      <.0001
*/

PROC MIXED DATA = height_long COVTEST NOCLPRINT METHOD=ML;
	CLASS id;
	MODEL hght = t1 t2/SOLUTION DDFM=BW;
	RANDOM INTERCEPT t1 /SUBJECT=id TYPE=UN GCORR;
RUN;
/*
                                 Covariance Parameter Estimates

                                                   Standard         Z
               Cov Parm     Subject    Estimate       Error     Value        Pr Z

               UN(1,1)      id           7.3118      1.2621      5.79      <.0001
               UN(2,1)      id           1.4896      0.4289      3.47      0.0005
               UN(2,2)      id           0.7894      0.2268      3.48      0.0003
               Residual                  2.6582      0.1838     14.46      <.0001


                                         Fit Statistics

                              -2 Log Likelihood              2501.3
                              AIC (smaller is better)        2515.3
                              AICC (smaller is better)       2515.5
                              BIC (smaller is better)        2532.2

                                   Solution for Fixed Effects

                                         Standard
                Effect       Estimate       Error      DF    t Value    Pr > |t|

                Intercept     83.0164      0.3240      82     256.20      <.0001
                t1            13.7843      0.1341     496     102.78      <.0001
                t2            -3.4294     0.08679     496     -39.51      <.0001
*/


*Log Time Model;
PROC MIXED DATA = height_long COVTEST NOCLPRINT METHOD=ML;
	CLASS id;
	MODEL hght = logt/SOLUTION DDFM=BW;
	RANDOM INTERCEPT logt/SUBJECT=id TYPE=UN GCORR;
RUN;

/*
                                 Covariance Parameter Estimates

                                                   Standard         Z
               Cov Parm     Subject    Estimate       Error     Value        Pr Z

               UN(1,1)      id                0           .       .         .
               UN(2,1)      id           1.2581      0.2764      4.55      <.0001
               UN(2,2)      id                0           .       .         .
               Residual                 12.4379      0.7590     16.39      <.0001

                                         Fit Statistics

                              -2 Log Likelihood              3212.1
                              AIC (smaller is better)        3220.1
                              AICC (smaller is better)       3220.2
                              BIC (smaller is better)        3229.8

                                   Solution for Fixed Effects

                                         Standard
                Effect       Estimate       Error      DF    t Value    Pr > |t|

                Intercept     49.6683      0.3344      82     148.55      <.0001
                logt          11.3892      0.1424     497      79.96      <.0001
*/

*SQRT Time Model;
PROC MIXED DATA = height_long COVTEST NOCLPRINT METHOD=ML;
	CLASS id;
	MODEL hght = sqrt/SOLUTION DDFM=BW;
	RANDOM INTERCEPT sqrt/SUBJECT=id TYPE=UN GCORR;
RUN;

/*
                                Covariance Parameter Estimates

                                                   Standard         Z
               Cov Parm     Subject    Estimate       Error     Value        Pr Z

               UN(1,1)      id           5.2111      0.9054      5.76      <.0001
               UN(2,1)      id          -0.2056      0.2004     -1.03      0.3049
               UN(2,2)      id           0.4204     0.08307      5.06      <.0001
               Residual                  0.8963     0.06200     14.46      <.0001


                                         Fit Statistics

                              -2 Log Likelihood              2039.7
                              AIC (smaller is better)        2051.7
                              AICC (smaller is better)       2051.9
                              BIC (smaller is better)        2066.3


                                   Solution for Fixed Effects

                                         Standard
                Effect       Estimate       Error      DF    t Value    Pr > |t|

                Intercept     54.8492      0.2642      82     207.59      <.0001
                sqrt           8.4254     0.08098     497     104.04      <.0001
*/
*Spline Model;
PROC MIXED DATA = height_long COVTEST NOCLPRINT METHOD=ML;
	CLASS id;
	MODEL hght = pre post/SOLUTION DDFM=BW;
	RANDOM INTERCEPT pre post/SUBJECT=id TYPE=UN GCORR;
RUN;

/*
                                 Covariance Parameter Estimates

                                                   Standard         Z
               Cov Parm     Subject    Estimate       Error     Value        Pr Z

               UN(1,1)      id           6.1245      1.0992      5.57      <.0001
               UN(2,1)      id           0.2333     0.09582      2.44      0.0149
               UN(2,2)      id          0.04363     0.01404      3.11      0.0009
               UN(3,1)      id          0.08389     0.02522      3.33      0.0009
               UN(3,2)      id         0.008394    0.002764      3.04      0.0024
               UN(3,3)      id         0.002174    0.001085      2.00      0.0226
               Residual                  1.8120      0.1355     13.37      <.0001


                                         Fit Statistics

                              -2 Log Likelihood              2339.2
                              AIC (smaller is better)        2359.2
                              AICC (smaller is better)       2359.6
                              BIC (smaller is better)        2383.4


                                   Solution for Fixed Effects

                                         Standard
                Effect       Estimate       Error      DF    t Value    Pr > |t|

                Intercept     73.3506      0.2938      82     249.67      <.0001
                pre            2.2267     0.03321     496      67.06      <.0001
                post           0.8874    0.009470     496      93.71      <.0001
*/


*Linear Model;
PROC NLMIXED DATA = height_long;
	b_1i = beta_1 + d_1i;
	b_2i = beta_2 + d_2i;

	traject = b_1i + b_2i * (age - 1);

	MODEL hght ~ NORMAL(traject, v_u);
	RANDOM d_1i d_2i ~ NORMAL([0,0], [v_1,
									  c_21, v_2])
	SUBJECT = id;
	PARMS	
		beta_1 = 40	beta_2 = 5
	
		v_1 = 70	v_2 = 1	c_21 = 0

		v_u = 10;
RUN;

** No Convergence **;

PROC NLMIXED DATA = height_long;
    b_1i = beta_1 + d_1i;
    b_2i = beta_2 + d_2i;
    b_3i = beta_3 + d_3i;

    traject = b_1i + b_2i * (age - 18)/12 + b_3i * ((age - 18)/12)**2;

    MODEL hght ~ NORMAL(traject, v_u);
    RANDOM d_1i d_2i d_3i ~ NORMAL([0,0,0], [v_1,
                                            c_21,  v_2,
                                            c_31, c_32, v_3])
    SUBJECT = id;
    PARMS
        beta_1 = 80  beta_2 = 15  beta_3 = -3
        v_1 = 10     v_2 = 1      v_3 = .1
        c_21 = 0     c_31 = 0     c_32 = 0
        v_u = 10; 
RUN;

** No Convergence **;

*Adjusted Quadratic Model;
PROC NLMIXED DATA = height_long;
    b_1i = beta_1 + d_1i;
    b_2i = beta_2 + d_2i;
    b_3i = beta_3;

    traject = b_1i + b_2i * (age - 18)/12 + b_3i * ((age - 18)/12)**2;

    MODEL hght ~ NORMAL(traject, v_u);
    RANDOM d_1i d_2i  ~ NORMAL([0,0], [v_1,
                                       c_21, v_2])
    SUBJECT = id OUT=quad_estimates;
    PARMS
        beta_1 = 80  beta_2 = 15  beta_3 = -3
        v_1 = 10     v_2 = 1      
        c_21 = 0     v_u = 10; 
RUN;

/*
                                         Fit Statistics

                            -2 Log Likelihood                 2501.3
                            AIC (smaller is better)           2515.3
                            AICC (smaller is better)          2515.5
                            BIC (smaller is better)           2532.2


                                      Parameter Estimates

                       Standard
  Parameter  Estimate     Error    DF  t Value  Pr > |t|   Alpha     Lower     Upper  Gradient

  beta_1      83.0164    0.3243    81   256.02    <.0001    0.05   82.3712   83.6616   0.00051
  beta_2      13.7843    0.1345    81   102.45    <.0001    0.05   13.5166   14.0520   -0.0016
  beta_3      -3.4294   0.08692    81   -39.45    <.0001    0.05   -3.6024   -3.2565   0.00006
  v_1          7.3119    1.2621    81     5.79    <.0001    0.05    4.8007    9.8230  0.000079
  v_2          0.7895    0.2269    81     3.48    0.0008    0.05    0.3381    1.2408  0.000898
  c_21         1.4896    0.4289    81     3.47    0.0008    0.05    0.6362    2.3431  -0.00041
  v_u          2.6582    0.1838    81    14.46    <.0001    0.05    2.2925    3.0239  0.000374
*/

* Individual Predicted Plot;
PROC SORT DATA = quad_estimates;
	BY id;
RUN;
DATA quad_estimates_b1;
	SET quad_estimates;
	IF effect = 'd_1i';
	b_1i = 83.0164 + estimate;
	KEEP id b_1i;
RUN;
DATA quad_estimates_b2;
	SET quad_estimates;
	IF effect = 'd_2i';
	b_2i = 13.7843 + estimate;
	KEEP id b_2i;
RUN;

DATA quad_estimates1;
	MERGE 	quad_estimates_b1 
			quad_estimates_b2; 
	BY id;
RUN;
 
*Creating a longitudinal database with predictions;
DATA quad_prediction;
	MERGE height_long quad_estimates1;
	BY id;
	pred = b_1i + b_2i * (age - 18)/12 - 3.4294 * ((age - 18)/12)**2;
	resid = hght - pred;
RUN;
GOPTIONS
	FTEXT = ARIAL
	CTEXT = BLACK
	HTEXT = 1.5;
PROC GPLOT DATA = quad_prediction;
	TITLE1 'IGS Height by Age - Quadratic Predictions';
	SYMBOL
		Interpol=join
		Color=black
		Repeat=5000
		Value=dot
		Height=.5
		Width=1;
	AXIS1
		LABEL = (F=ARIAL H = 2 A=90 'Height (cm)')
		ORDER = (40 to 110 BY 10)
		MINOR = none
		OFFSET= (5 pct 5 pct);
	AXIS2
		LABEL = (F=ARIAL H = 2 'Age in Months')	
		ORDER = (0 to 40 BY 10)
		MINOR = none
		OFFSET= (5 PCT, 5 PCT);
	PLOT pred * age = id/NOLEGEND VAXIS= AXIS1 HAXIS=AXIS2;
RUN;

DATA quad_prediction1;
	SET quad_estimates1;
	DO age = 1 to 36 by .5;
		pred = b_1i + b_2i * (age - 18)/12 - 3.4294 * ((age - 18)/12)**2;
		OUTPUT;
	END;
RUN;


PROC GPLOT DATA = quad_prediction1;
	TITLE1 'IGS Height by Age - Quadratic Predictions';
	SYMBOL
		Interpol=join
		Color=black
		Repeat=5000
		Value=none
		Height=.5
		Width=1;
	AXIS1
		LABEL = (F=ARIAL H = 2 A=90 'Height (cm)')
		ORDER = (40 to 110 BY 10)
		MINOR = none
		OFFSET= (5 pct 5 pct);
	AXIS2
		LABEL = (F=ARIAL H = 2 'Age in Months')	
		ORDER = (0 to 40 BY 10)
		MINOR = none
		OFFSET= (5 PCT, 5 PCT);
	PLOT pred * age = id/NOLEGEND VAXIS= AXIS1 HAXIS=AXIS2;
RUN;


*Log Time Model;
PROC NLMIXED DATA = height_long;
	b_1i = beta_1 + d_1i;
	b_2i = beta_2 + d_2i;

	traject = b_1i + b_2i * LOG(age);

	MODEL hght ~ NORMAL(traject, v_u);
	RANDOM d_1i d_2i ~ NORMAL([0,0], [v_1,
								     c_21,  v_2])
	SUBJECT = id;
	PARMS	
		beta_1 = 40	beta_2 = 15 
	
		v_1 = 10	v_2 = 1		
		c_21 = 0	

		v_u = 10;
RUN;

*Square Root Model;
PROC NLMIXED DATA = height_long;
	b_1i = beta_1 + d_1i;
	b_2i = beta_2 + d_2i;

	traject = b_1i + b_2i * (SQRT(age) - 1);

	MODEL hght ~ NORMAL(traject, v_u);
	RANDOM d_1i d_2i ~ NORMAL([0,0], [v_1,
								     c_21,  v_2])
	SUBJECT = id;
	PARMS	
		beta_1 = 40	beta_2 = 15 
	
		v_1 = 10	v_2 = 1		
		c_21 = 0	

		v_u = 10;
RUN;

/*
                                         Fit Statistics

                            -2 Log Likelihood                 2039.7
                            AIC (smaller is better)           2051.7
                            AICC (smaller is better)          2051.9
                            BIC (smaller is better)           2066.3

                                      The NLMIXED Procedure

                                      Parameter Estimates

                       Standard
  Parameter  Estimate     Error    DF  t Value  Pr > |t|   Alpha     Lower     Upper  Gradient

  beta_1      54.8492    0.2643    81   207.49    <.0001    0.05   54.3232   55.3752  -0.00013
  beta_2       8.4254   0.08114    81   103.84    <.0001    0.05    8.2640    8.5868  -0.00216
  v_1          5.2111    0.9054    81     5.76    <.0001    0.05    3.4096    7.0126  0.000054
  v_2          0.4204   0.08308    81     5.06    <.0001    0.05    0.2551    0.5857  0.003463
  c_21        -0.2055    0.2004    81    -1.03    0.3081    0.05   -0.6042    0.1931  0.001538
  v_u          0.8963   0.06200    81    14.46    <.0001    0.05    0.7729    1.0196  0.000164
*/

PROC NLMIXED DATA = height_long;
	b_1i = beta_1 + d_1i;
	b_2i = beta_2 + d_2i;

	traject = b_1i + b_2i * (SQRT(age-1));

	MODEL hght ~ NORMAL(traject, v_u);
	RANDOM d_1i d_2i ~ NORMAL([0,0], [v_1,
								     c_21,  v_2])
	SUBJECT = id;
	PARMS	
		beta_1 = 40	beta_2 = 15 
	
		v_1 = 10	v_2 = 1		
		c_21 = 0	

		v_u = 10;
RUN;

/*
                                         Fit Statistics

                            -2 Log Likelihood                 2493.4
                            AIC (smaller is better)           2505.4
                            AICC (smaller is better)          2505.6
                            BIC (smaller is better)           2519.9


                                      Parameter Estimates

                       Standard
  Parameter  Estimate     Error    DF  t Value  Pr > |t|   Alpha     Lower     Upper  Gradient

  beta_1      51.8843    0.2730    81   190.03    <.0001    0.05   51.3411   52.4276  -0.00008
  beta_2       7.3121   0.07162    81   102.10    <.0001    0.05    7.1696    7.4545  0.000305
  v_1          4.3486    0.9627    81     4.52    <.0001    0.05    2.4332    6.2640  -0.00011
  v_2          0.2447   0.06443    81     3.80    0.0003    0.05    0.1165    0.3729  -0.00042
  c_21       -0.06203    0.1896    81    -0.33    0.7443    0.05   -0.4392    0.3151  -0.00028
  v_u          2.5866    0.1783    81    14.50    <.0001    0.05    2.2318    2.9415  -0.00002
*/

*Bi-Linear Spline;
PROC NLMIXED DATA = height_long;
    b_1i = beta_1 + d_1i;
    b_2i = beta_2 + d_2i;
    b_3i = beta_3 + d_3i;

	traject = b_1i + b_2i * MIN(0,age-9) + b_3i * MAX(0,age-9);

	MODEL hght ~ NORMAL(traject, v_u);
	RANDOM d_1i d_2i d_3i ~ NORMAL([0,0,0], [v_1,
									        c_21,  v_2,
                                            c_31, c_32, v_3])
	SUBJECT = id OUT = sp_estimates;
	PARMS	
		beta_1 = 60	beta_2 = 2 	beta_3 = 1
	
		v_1 = 5		v_2 = .05	v_3 = .2
		c_21 = 0	c_31 = 0	c_32 = 0

		v_u = 2;
RUN;

/*
                                         Fit Statistics

                            -2 Log Likelihood                 2339.2
                            AIC (smaller is better)           2359.2
                            AICC (smaller is better)          2359.6
                            BIC (smaller is better)           2383.4


                                      Parameter Estimates

                       Standard
  Parameter  Estimate     Error    DF  t Value  Pr > |t|   Alpha     Lower     Upper  Gradient

  beta_1      73.3507    0.2939    80   249.55    <.0001    0.05   72.7657   73.9356   0.00079
  beta_2       2.2267   0.03335    80    66.77    <.0001    0.05    2.1603    2.2931  -0.00011
  beta_3       0.8874  0.009502    80    93.40    <.0001    0.05    0.8685    0.9063  -0.00732
  v_1          6.1244    1.0991    80     5.57    <.0001    0.05    3.9370    8.3117  -0.00006
  v_2         0.04363   0.01404    80     3.11    0.0026    0.05   0.01568   0.07157  0.007665
  v_3        0.002174  0.001085    80     2.00    0.0486    0.05  0.000014  0.004334   0.18495
  c_21         0.2333   0.09581    80     2.44    0.0171    0.05   0.04263    0.4240  -0.00225
  c_31        0.08389   0.02522    80     3.33    0.0013    0.05   0.03371    0.1341  -0.00222
  c_32       0.008394  0.002764    80     3.04    0.0032    0.05  0.002894   0.01389  0.026482
  v_u          1.8120    0.1355    80    13.37    <.0001    0.05    1.5423    2.0818  0.000582
*/

* Individual Predicted Plot;
PROC SORT DATA = sp_estimates;
	BY id;
RUN;
DATA sp_estimates_b1;
	SET sp_estimates;
	IF effect = 'd_1i';
	b_1i = 73.3506 + estimate;
	KEEP id b_1i;
RUN;
DATA sp_estimates_b2;
	SET sp_estimates;
	IF effect = 'd_2i';
	b_2i = 2.2267 + estimate;
	KEEP id b_2i;
RUN;
DATA sp_estimates_b3;
	SET sp_estimates;
	IF effect = 'd_3i';
	b_3i = 0.8874 + estimate;
	KEEP id b_3i;
RUN;

DATA sp_estimates1;
	MERGE 	sp_estimates_b1 
			sp_estimates_b2 
			sp_estimates_b3; 
	BY id;
RUN;
 
*Creating a longitudinal database with predictions;
DATA sp_prediction;
	MERGE height_long sp_estimates1;
	BY id;
	pred = b_1i + b_2i * MIN(0,age-9) + b_3i * MAX(0,age-9);
	resid = hght - pred;
RUN;
GOPTIONS
	FTEXT = ARIAL
	CTEXT = BLACK
	HTEXT = 1.5;
PROC GPLOT DATA = sp_prediction;
	TITLE1 'IGS Height by Age - Bi-Linear Predictions';
	SYMBOL
		Interpol=join
		Color=black
		Repeat=5000
		Value=dot
		Height=.5
		Width=1;
	AXIS1
		LABEL = (F=ARIAL H = 2 A=90 'Height (cm)')
		ORDER = (40 to 110 BY 10)
		MINOR = none
		OFFSET= (5 pct 5 pct);
	AXIS2
		LABEL = (F=ARIAL H = 2 'Age in Months')	
		ORDER = (0 to 40 BY 10)
		MINOR = none
		OFFSET= (5 PCT, 5 PCT);
	PLOT pred * age = id/NOLEGEND VAXIS= AXIS1 HAXIS=AXIS2;
RUN;

DATA sp_prediction1;
	SET sp_estimates1;
	DO age = 1 to 36 by .5;
		pred = b_1i + b_2i * MIN(0,age-9) + b_3i * MAX(0,age-9);
		OUTPUT;
	END;
RUN;


PROC GPLOT DATA = sp_prediction1;
	TITLE1 'IGS Height by Age - Bi_Linear Predictions';
	SYMBOL
		Interpol=join
		Color=black
		Repeat=5000
		Value=none
		Height=.5
		Width=1;
	AXIS1
		LABEL = (F=ARIAL H = 2 A=90 'Height (cm)')
		ORDER = (40 to 110 BY 10)
		MINOR = none
		OFFSET= (5 pct 5 pct);
	AXIS2
		LABEL = (F=ARIAL H = 2 'Age in Months')	
		ORDER = (0 to 40 BY 10)
		MINOR = none
		OFFSET= (5 PCT, 5 PCT);
	PLOT pred * age = id/NOLEGEND VAXIS= AXIS1 HAXIS=AXIS2 HREF = 9;
RUN;


